from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.test import override_settings
from faker import Faker

from edc_auth.auth_objects import (
    ACCOUNT_MANAGER_ROLE,
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    CUSTOM_ROLE,
    NURSE_ROLE,
    STAFF_ROLE,
)
from edc_auth.auth_objects.default_role_names import STATISTICIAN
from edc_auth.auth_updater import AuthUpdater
from edc_auth.models import Role
from edc_auth.site_auths import site_auths

from ..utils import EdcAuthTestCase, create_users

fake = Faker()

site_names = ["harare", "gaborone", "kampala"]
user_model = get_user_model()


@override_settings(
    EDC_AUTH_SKIP_SITE_AUTHS=False,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
)
class TestRoles(EdcAuthTestCase):
    def test_post_migrate(self):
        """Assert post-migrate updated defaults for model Role."""
        AuthUpdater(verbose=False)
        self.assertGreater(Role.objects.all().count(), 0)
        for role_name in [
            ACCOUNT_MANAGER_ROLE,
            AUDITOR_ROLE,
            CLINICIAN_ROLE,
            CLINICIAN_SUPER_ROLE,
            CUSTOM_ROLE,
            NURSE_ROLE,
            STAFF_ROLE,
            STATISTICIAN,
        ]:
            try:
                Role.objects.get(name=role_name)
            except ObjectDoesNotExist:
                self.fail(f"Role name unexpectedly does not exist. Got {role_name}")

        role = Role.objects.get(name=CLINICIAN_ROLE)
        groups_from_role = [group.name for group in role.groups.all()]
        groups_from_role.sort()
        groups_from_site_auths = site_auths.roles.get(CLINICIAN_ROLE)
        groups_from_site_auths.sort()
        self.assertEqual(groups_from_role, groups_from_site_auths)

    def test_add_roles_to_user(self):
        AuthUpdater(verbose=False)
        create_users()
        user = user_model.objects.all()[0]
        self.assertEqual(user.groups.all().count(), 0)
        role = Role.objects.get(name=CLINICIAN_ROLE)
        # should trigger post add m2m signal
        user.userprofile.roles.add(role)
        user.refresh_from_db()
        self.assertGreater(user.groups.all().count(), 0)
        # see groups_by_role_name for expected group counts
        # note, count is the unique list count
        self.assertEqual(user.groups.all().count(), len(site_auths.roles.get(CLINICIAN_ROLE)))

    def test_remove_roles_to_user(self):
        AuthUpdater(verbose=False)
        create_users()
        user = user_model.objects.all()[0]
        self.assertEqual(user.groups.all().count(), 0)
        clinician_role = Role.objects.get(name=CLINICIAN_ROLE)
        clinician_super_role = Role.objects.get(name=CLINICIAN_SUPER_ROLE)
        clinician_groups = site_auths.roles.get(CLINICIAN_ROLE)
        clinician_super_groups = site_auths.roles.get(CLINICIAN_SUPER_ROLE)
        unique_group_cnt = len(list(set(clinician_groups + clinician_super_groups)))
        user.userprofile.roles.add(clinician_role)
        user.userprofile.roles.add(clinician_super_role)
        user.refresh_from_db()
        self.assertEqual(user.groups.all().count(), unique_group_cnt)
        # should trigger post remove m2m signal
        user.userprofile.roles.remove(clinician_super_role)
        self.assertEqual(user.groups.all().count(), len(clinician_groups))



import csv
import os
from importlib import import_module
from secrets import choice
from tempfile import mkdtemp

from django.contrib.auth.models import Group, User
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from edc_lab.auth_objects import LAB_TECHNICIAN_ROLE
from edc_randomization.randomizer import Randomizer
from edc_randomization.site_randomizers import site_randomizers
from edc_sites.models import SiteProfile
from faker import Faker
from mempass import mkpassword

from edc_auth.auth_objects import CLINICIAN_ROLE
from edc_auth.site_auths import site_auths
from edc_auth.tests.randomizers import CustomRandomizer

from ..import_users import fieldnames

fake = Faker()
group_names = ["CLINIC", "LAB"]
sites = {10: "harare", 20: "gaborone", 30: "kampala"}


class EdcAuthTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        SiteProfile.objects.all().delete()
        Site.objects.all().delete()
        for site_id, site_name in sites.items():
            Site.objects.create(id=site_id, name=site_name, domain=f"{site_name}.example.com")
        site_randomizers._registry = {}
        site_randomizers.register(Randomizer)
        site_randomizers.register(CustomRandomizer)
        site_auths.initialize()
        import_module("edc_navbar.auths")
        import_module("edc_dashboard.auths")
        import_module("edc_review_dashboard.auths")
        import_module("edc_randomization.auths")


def create_users(count=None, group_name=None, site_name=None):
    usernames = []
    for _ in range(0, count or 2):
        first_name = fake.first_name()
        last_name = fake.last_name()
        username = (first_name[0] + last_name).lower()
        user_data = {
            "username": username,
            "first_name": first_name,
            "last_name": last_name,
            "email": fake.email(),
            "is_active": True,
            "is_staff": True,
            "is_superuser": False,
        }
        user = User.objects.create(**user_data)
        user.userprofile.job_title = "Research Assistant"
        site = Site.objects.get(name=site_name or choice([v for v in sites.values()]))
        user.userprofile.sites.add(site)
        user.userprofile.save()
        if group_name:
            try:
                group = Group.objects.get(name__iexact=group_name)
            except ObjectDoesNotExist:
                group = Group.objects.create(name=group_name)
            user.groups.add(group)
            user.save()
        usernames.append(user.username)
    return usernames


def create_user_csv_file(
    user_count=None,
    filename=None,
    include_passwords=None,
):
    folder = mkdtemp()
    filename = filename or os.path.join(folder, "users.csv")
    with open(filename, "w") as f:
        writer = csv.DictWriter(f, fieldnames=fieldnames, delimiter="|")
        writer.writeheader()
        for _ in range(0, user_count or 2):
            first_name = fake.first_name()
            last_name = fake.last_name()
            username = f"{first_name[0]}{''.join(last_name.split(' '))}".lower()
            writer.writerow(
                {
                    "username": username,
                    "password": mkpassword(3) if include_passwords else None,
                    "first_name": first_name,
                    "last_name": last_name,
                    "is_staff": True,
                    "is_active": True,
                    "job_title": "Research Assistant",
                    "email": fake.email(),
                    "alternate_email": fake.email(),
                    "mobile": fake.phone_number(),
                    "sites": choice([v for v in sites.values()]),
                    "roles": f"{CLINICIAN_ROLE},{LAB_TECHNICIAN_ROLE}",
                }
            )
    return filename



# Generated by Django 2.1 on 2018-08-03 03:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0002_auto_20180803_0515")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="sites",
            field=models.ManyToManyField(blank=True, to="sites.Site"),
        )
    ]



from tempfile import mkdtemp

from edc_randomization.randomizer import Randomizer

tmpdir = mkdtemp()


class CustomRandomizer(Randomizer):
    name = "custom_randomizer"
    model = "edc_auth.customrandomizationlist"
    randomization_list_path = tmpdir



# Generated by Django 2.2.6 on 2019-10-21 22:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0007_auto_20181025_0150")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="job_title",
            field=models.CharField(blank=True, max_length=100, null=True),
        )
    ]



from django import forms
from django.conf import settings
from django.contrib.auth.forms import UserChangeForm as BaseForm
from django.utils.html import format_html
from edc_notification.utils import get_email_enabled
from edc_randomization.auth_objects import RANDO_UNBLINDED
from edc_randomization.blinding import (
    is_blinded_user,
    raise_if_prohibited_from_unblinded_rando_group,
)

from .models import UserProfile


class UserChangeForm(BaseForm):
    def clean(self):
        cleaned_data = super().clean()
        if not self.cleaned_data.get("first_name"):
            raise forms.ValidationError({"first_name": "Required"})
        if not self.cleaned_data.get("last_name"):
            raise forms.ValidationError({"last_name": "Required"})
        if not self.cleaned_data.get("email"):
            raise forms.ValidationError({"email": "Required"})
        qs = self.cleaned_data.get("groups")
        if qs and qs.count() > 0:
            raise_if_prohibited_from_unblinded_rando_group(self.instance.username, qs)
        return cleaned_data


class UserProfileForm(forms.ModelForm):
    def clean(self):
        cleaned_data = super().clean()

        qs = self.cleaned_data.get("email_notifications")
        if qs and qs.count() > 0:
            if not get_email_enabled():
                raise forms.ValidationError(
                    {
                        "email_notifications": (
                            "You may not choose an email "
                            "notification. Email is not enabled. "
                            "Contact your EDC administrator."
                        )
                    }
                )
        qs = self.cleaned_data.get("sms_notifications")
        if qs and qs.count() > 0:
            if not settings.TWILIO_ENABLED:
                raise forms.ValidationError(
                    {
                        "sms_notifications": (
                            "You may not choose an SMS "
                            "notification. SMS is not enabled. "
                            "Contact your EDC administrator."
                        )
                    }
                )
        qs = self.cleaned_data.get("roles")
        if qs and qs.count() > 0:
            for role in qs:
                if RANDO_UNBLINDED in [
                    obj.name for obj in role.groups.all()
                ] and is_blinded_user(self.instance.user.username):
                    raise forms.ValidationError(
                        {
                            "roles": format_html(
                                "This user is not unblinded and may not be assigned "
                                "the role of <U>{}</U>.",
                                role.name.title(),
                            )
                        }
                    )
        return cleaned_data

    class Meta:
        model = UserProfile
        fields = "__all__"



ACCOUNT_MANAGER_ROLE = "account_manager"
AUDITOR_ROLE = "auditor"
CLINICIAN_ROLE = "clinician"
CLINICIAN_SUPER_ROLE = "clinician_super"
CUSTOM_ROLE = "custom"
NURSE_ROLE = "research_nurse"
STAFF_ROLE = "staff"
STATISTICIAN = "statistician"



# Generated by Django 3.1.5 on 2021-01-25 01:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_auth", "0020_auto_20210120_0005"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="export_format",
            field=models.CharField(
                blank=True,
                choices=[("CSV", "CSV")],
                default="CSV",
                help_text="Note: requires export permissions",
                max_length=25,
                null=True,
                verbose_name="Export format",
            ),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-26 18:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0014_auto_20191026_1841")]

    operations = [
        migrations.AlterModelOptions(
            name="role", options={"ordering": ["display_index", "display_name"]}
        ),
        migrations.RemoveIndex(model_name="role", name="edc_auth_ro_id_587a9b_idx"),
        migrations.RemoveField(model_name="role", name="field_name"),
        migrations.RemoveField(model_name="role", name="version"),
        migrations.RenameField(model_name="role", old_name="name", new_name="display_name"),
        migrations.AddIndex(
            model_name="role",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="edc_auth_ro_id_cc6bf4_idx",
            ),
        ),
    ]



import sys

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand, CommandError

from edc_auth.password_setter import PasswordSetter


class Command(BaseCommand):

    help = "Reset a user's password"

    def add_arguments(self, parser):
        parser.add_argument("username")
        parser.add_argument("super_user")
        parser.add_argument(
            "--resource",
            default="clinicedc.org",
            action="store_true",
            dest="resource",
            help="Resource name",
        )

        parser.add_argument(
            "--email", action="store_true", dest="email", help="Alternate email"
        )

    def handle(self, *args, **options):
        try:
            user = User.objects.get(username=options["username"])
        except ObjectDoesNotExist as e:
            raise CommandError(e)
        else:
            p = PasswordSetter(super_username=options["super_user"])
            p.reset_user(username=options["username"])
            sys.stdout.write(f"\nYour password has been reset and emailed to {user.email}\n")



import sys
from typing import Callable, List, Optional, Tuple

from django.apps import apps as django_apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.core.management.color import color_style

from ..site_auths import site_auths
from .group_updater import GroupUpdater
from .role_updater import RoleUpdater

style = color_style()


class AuthUpdater:
    group_updater_cls = GroupUpdater
    role_updater_cls = RoleUpdater

    def __init__(
        self,
        groups: Optional[dict] = None,
        roles: Optional[dict] = None,
        pii_models: Optional[list] = None,
        pre_update_funcs: Optional[list] = None,
        post_update_funcs: Optional[List[Tuple[str, Callable]]] = None,
        custom_permissions_tuples: Optional[dict] = None,
        verbose=None,
        apps=None,
        warn_only=None,
    ):
        site_auths.verify_and_populate()
        custom_permissions_tuples = (
            custom_permissions_tuples or site_auths.custom_permissions_tuples
        )
        groups = groups or site_auths.groups
        pii_models = pii_models or site_auths.pii_models
        post_update_funcs = post_update_funcs or site_auths.post_update_funcs
        pre_update_funcs = pre_update_funcs or site_auths.pre_update_funcs
        roles = roles or site_auths.roles
        self.apps = apps
        if not self.edc_auth_skip_auth_updater:
            self.verbose = verbose
            self.apps = apps
            if self.verbose:
                sys.stdout.write(style.MIGRATE_HEADING("Updating groups and permissions:\n"))
            self.group_updater = self.group_updater_cls(
                groups=groups,
                pii_models=pii_models,
                custom_permissions_tuples=custom_permissions_tuples,
                verbose=self.verbose,
                apps=self.apps,
                warn_only=warn_only,
            )
            self.role_updater = self.role_updater_cls(
                roles=roles,
                verbose=self.verbose,
            )
            self.run_pre_updates(pre_update_funcs)
            self.group_updater.create_custom_permissions_from_tuples()
            self.groups = self.group_updater.update_groups()
            self.roles = self.role_updater.update_roles()
            self.run_post_updates(post_update_funcs)
            self.refresh_groups_in_roles_per_user()
            if verbose:
                sys.stdout.write(style.MIGRATE_HEADING("Done.\n"))
                sys.stdout.flush()

    def __repr__(self):
        return (
            f"{self.__class__.__name__}(edc_auth_skip_auth_updater="
            f"{self.edc_auth_skip_auth_updater})"
        )

    @property
    def edc_auth_skip_auth_updater(self):
        return getattr(settings, "EDC_AUTH_SKIP_AUTH_UPDATER", False)

    def run_pre_updates(self, pre_updates):
        """Custom funcs that operate after all groups and roles have been created"""
        if self.verbose:
            sys.stdout.write(style.MIGRATE_HEADING(" - Running pre updates:\n"))
        if pre_updates:
            for func in pre_updates:
                sys.stdout.write(f"   * {func.__name__}\n")
                func(self)
        else:
            if self.verbose:
                sys.stdout.write("   * nothing to do\n")
        if self.verbose:
            sys.stdout.write("   Done.\n")

    def run_post_updates(self, post_updates: List[Tuple[str, Callable]]):
        """Custom funcs that operate after all groups and roles have been created"""
        if self.verbose:
            sys.stdout.write(style.MIGRATE_HEADING(" - Running post updates:\n"))
        if post_updates:
            for app_label, func in post_updates:
                sys.stdout.write(f"   * {func.__name__}({app_label})\n")
                func(self, app_label)
        else:
            if self.verbose:
                sys.stdout.write("   * nothing to do\n")
        if self.verbose:
            sys.stdout.write("   Done.\n")

    def create_permissions_from_tuples(self, **kwargs):
        return self.group_updater.create_permissions_from_tuples(**kwargs)

    @property
    def group_model_cls(self):
        return self.group_updater.group_model_cls

    def remove_permissions_by_codenames(self, **kwargs):
        return self.group_updater.remove_permissions_by_codenames(**kwargs)

    @classmethod
    def add_empty_groups_for_tests(cls, *extra_names, apps=None):
        """Adds group names without codenames.

        For tests
        """
        apps = apps or django_apps
        groups_names = extra_names + tuple(site_auths.groups)
        for name in groups_names:
            try:
                apps.get_model("auth.group").objects.get(name=name)
            except ObjectDoesNotExist:
                apps.get_model("auth.group").objects.create(name=name)
                site_auths.groups.update({name: []})
        return groups_names

    @classmethod
    def add_empty_roles_for_tests(cls, *extra_names, apps=None):
        """Adds roles names without groups.

        For tests
        """
        apps = apps or django_apps
        role_names = extra_names + tuple(site_auths.roles)
        for name in role_names:
            display_name = name.replace("_", " ").lower().title()
            try:
                role_obj = apps.get_model("edc_auth.role").objects.get(name=name)
            except ObjectDoesNotExist:
                apps.get_model("edc_auth.role").objects.create(
                    name=name, display_name=display_name
                )
                site_auths.roles.update({name: []})
            else:
                role_obj.display_name = display_name
                role_obj.save()
        return role_names

    @staticmethod
    def refresh_groups_in_roles_per_user():
        """Clear then add back roles to trigger post-save signal."""
        for user in get_user_model().objects.all():
            roles = [obj for obj in user.userprofile.roles.all()]
            user.userprofile.roles.clear()
            user.groups.clear()
            for role in roles:
                user.userprofile.roles.add(role)
            user.userprofile.save()
            user.save()



from typing import Dict, List

from edc_auth.auth_objects.default_role_names import STATISTICIAN

from .default_group_names import (
    ACCOUNT_MANAGER,
    ADMINISTRATION,
    AUDITOR,
    CLINIC,
    CLINIC_SUPER,
    EVERYONE,
    PII,
    PII_VIEW,
)
from .default_role_names import (
    ACCOUNT_MANAGER_ROLE,
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    CUSTOM_ROLE,
    NURSE_ROLE,
    STAFF_ROLE,
)

# Format {ROLE_NAME: [GROUP_NAME, GROUP_NAME, ...]
default_roles: Dict[str, List[str]] = {
    ACCOUNT_MANAGER_ROLE: [ACCOUNT_MANAGER],
    AUDITOR_ROLE: [
        AUDITOR,
        PII_VIEW,
    ],
    CLINICIAN_ROLE: [
        CLINIC,
        PII,
    ],
    CLINICIAN_SUPER_ROLE: [
        CLINIC_SUPER,
        CLINIC,
        PII,
    ],
    CUSTOM_ROLE: [],
    NURSE_ROLE: [
        CLINIC,
        PII,
    ],
    STAFF_ROLE: [ADMINISTRATION, EVERYONE],
    STATISTICIAN: [AUDITOR],
}



# Generated by Django 2.1 on 2018-08-04 20:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0004_auto_20180804_2041")]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="job_title",
            field=models.CharField(blank=True, max_length=10, null=True),
        )
    ]



from django.contrib import admin
from django.contrib.admin.models import LogEntry
from edc_model_admin.mixins import TemplatesModelAdminMixin
from logentry_admin.admin import ActionListFilter, LogEntryAdmin, UserListFilter

from edc_auth.admin_site import edc_auth_admin

admin.site.unregister(LogEntry)


@admin.register(LogEntry, site=edc_auth_admin)
class LogEntriesAdmin(TemplatesModelAdminMixin, LogEntryAdmin):

    list_filter = (
        "action_time",
        ActionListFilter,
        UserListFilter,
        "content_type",
    )



# Generated by Django 2.2.6 on 2019-10-26 18:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0015_auto_20191026_2149")]

    operations = [
        migrations.RenameField(model_name="role", old_name="short_name", new_name="name"),
        migrations.AlterField(
            model_name="role",
            name="display_name",
            field=models.CharField(
                db_index=True,
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Display Name",
            ),
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_randomization.model_mixins import RandomizationListModelMixin


class PiiModel(models.Model):

    name = models.CharField(max_length=50, null=True)

    class Meta:
        permissions = (("be_happy", "Can be happy"),)


class AuditorModel(models.Model):

    name = models.CharField(max_length=50, null=True)

    class Meta:
        permissions = (("be_sad", "Can be sad"),)


class TestModel(BaseUuidModel):

    name = models.CharField(max_length=50, null=True)


class SubjectRequisition(BaseUuidModel):

    name = models.CharField(max_length=50, null=True)

    history = HistoricalRecords()


class SubjectConsent(models.Model):

    name = models.CharField(max_length=50, null=True)

    history = HistoricalRecords()


class SubjectReconsent(models.Model):

    name = models.CharField(max_length=50, null=True)

    history = HistoricalRecords()


class CustomRandomizationList(RandomizationListModelMixin, BaseUuidModel):

    pass



# Generated by Django 3.2.13 on 2022-08-24 21:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_auth", "0025_permissions"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Permissions",
            new_name="EdcPermissions",
        ),
    ]



# Generated by Django 2.0.7 on 2018-07-31 21:01

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="UserProfile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("study_site", models.CharField(blank=True, max_length=100, null=True)),
                (
                    "study_country",
                    models.CharField(
                        blank=True,
                        help_text="user's country of work",
                        max_length=100,
                        null=True,
                    ),
                ),
                (
                    "country",
                    models.CharField(
                        blank=True,
                        help_text="user's country of origin",
                        max_length=100,
                        null=True,
                    ),
                ),
                (
                    "clinic_label_printer",
                    models.CharField(
                        blank=True,
                        help_text='Change in <a href="/edc_label/">Edc Label Administration</a>',
                        max_length=100,
                        null=True,
                    ),
                ),
                (
                    "lab_label_printer",
                    models.CharField(
                        blank=True,
                        help_text='Change in <a href="/edc_label/">Edc Label Administration</a>',
                        max_length=100,
                        null=True,
                    ),
                ),
                (
                    "print_server",
                    models.CharField(
                        blank=True,
                        help_text='Change in <a href="/edc_label/">Edc Label Administration</a>',
                        max_length=100,
                        null=True,
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        )
    ]



from django.conf import settings
from django.contrib import messages
from django.contrib.auth.views import LoginView as BaseLoginView
from django_revision.revision import site_revision
from edc_dashboard.utils import get_bootstrap_version, get_template_path_with_bootstrap
from edc_protocol import Protocol


class LoginView(BaseLoginView):
    template_name = f"edc_auth/bootstrap{get_bootstrap_version()}/login.html"

    def get_context_data(self, **kwargs):
        """Tests cookies."""
        context = super().get_context_data(**kwargs)
        context.update(**self.extra_context)
        self.request.session.set_test_cookie()
        if not self.request.session.test_cookie_worked():
            messages.add_message(self.request, messages.ERROR, "Please enable cookies.")
        self.request.session.delete_test_cookie()
        return context

    @property
    def extra_context(self):
        protocol = Protocol()
        try:
            live_system = settings.LIVE_SYSTEM
        except AttributeError:
            live_system = None
        try:
            allow_password_reset = settings.ALLOW_PASSWORD_RESET
        except AttributeError:
            allow_password_reset = None
        return {
            "edc_base_template": get_template_path_with_bootstrap("edc_dashboard/base.html"),
            "DEBUG": settings.DEBUG,
            "ALLOW_PASSWORD_RESET": allow_password_reset,
            "copyright": protocol.copyright,
            "disclaimer": protocol.disclaimer,
            "institution": protocol.institution,
            "license": protocol.license,
            "revision": site_revision.tag,
            "project_name": protocol.project_name,
            "live_system": live_system,
            "INDEX_PAGE": getattr(settings, "INDEX_PAGE", None),
            "INDEX_PAGE_LABEL": getattr(settings, "INDEX_PAGE_LABEL", None),
        }



# Generated by Django 3.2.13 on 2022-08-25 01:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_auth", "0026_rename_permissions_edcpermissions"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="edcpermissions",
            options={
                "verbose_name": "Edc Permissions",
                "verbose_name_plural": "Edc Permissions",
            },
        ),
    ]



ACCOUNT_MANAGER = "ACCOUNT_MANAGER"
ADMINISTRATION = "ADMINISTRATION"
AUDITOR = "AUDITOR"
CELERY_MANAGER = "CELERY_MANAGER"
CLINIC = "CLINIC"
CLINIC_SUPER = "CLINIC_SUPER"
EVERYONE = "EVERYONE"
PII = "PII"
PII_VIEW = "PII_VIEW"



from .codenames import account_manager, administration, celery_manager, everyone
from .default_group_names import (
    ACCOUNT_MANAGER,
    ADMINISTRATION,
    AUDITOR,
    CELERY_MANAGER,
    CLINIC,
    CLINIC_SUPER,
    EVERYONE,
    PII,
    PII_VIEW,
)

default_groups = {
    ACCOUNT_MANAGER: account_manager,
    ADMINISTRATION: administration,
    AUDITOR: [],
    CELERY_MANAGER: celery_manager,
    CLINIC: [],
    CLINIC_SUPER: [],
    EVERYONE: everyone,
    PII: [],
    PII_VIEW: [],
}



from .codenames import (
    account_manager,
    administration,
    auditor,
    celery_manager,
    clinic,
    clinic_super,
    everyone,
)
from .default_group_names import (
    ACCOUNT_MANAGER,
    ADMINISTRATION,
    AUDITOR,
    CELERY_MANAGER,
    CLINIC,
    CLINIC_SUPER,
    EVERYONE,
    PII,
    PII_VIEW,
)
from .default_groups import default_groups
from .default_pii_models import default_pii_models
from .default_role_names import (
    ACCOUNT_MANAGER_ROLE,
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    CUSTOM_ROLE,
    NURSE_ROLE,
    STAFF_ROLE,
)
from .default_roles import default_roles



# Generated by Django 2.2.6 on 2019-10-29 17:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0017_auto_20191026_2231")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="roles",
            field=models.ManyToManyField(blank=True, to="edc_auth.Role"),
        )
    ]



import csv
import os
import re
from datetime import datetime
from string import Template
from typing import Any, List, Optional

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import EmailMessage
from edc_protocol import Protocol
from mempass import PasswordGenerator

from .auth_objects import ACCOUNT_MANAGER_ROLE, STAFF_ROLE
from .export_users import export_users
from .models import Role

required_role_names = {STAFF_ROLE: "Staff"}


class UserImporterError(Exception):
    pass


add_user_template = Template(
    "Hi $first_name, \n\n"
    "Your $resource_name user account has been created.\n\n"
    "Your username is $username.\n\n"
    "Your password is:\n\n$password\n\n"
    "(Yes, that is your password)\n\n"
    "You are authorized to log into the following sites:\n\n - $site_names.\n\n"
    "As a $job_title, you have been assigned the following roles:\n\n - $role_names.\n\n"
    "Thanks.\n\n"
    "$project_name"
)

change_user_template = Template(
    "Hi $first_name, \n\n"
    "Your `$resource_name` user account has been updated.\n\n"
    "Your username is `$username`.\n\n"
    "Your new password is:\n\n$password\n\n"
    "You are authorized to log into the following sites:\n\n - $site_names.\n\n"
    "As a $job_title, you have been assigned the following roles:\n\n - $role_names.\n\n"
    "Thanks.\n\n"
    "$project_name"
)


fieldnames = [
    "username",
    "password",
    "first_name",
    "last_name",
    "is_superuser",
    "is_staff",
    "is_active",
    "job_title",
    "email",
    "alternate_email",
    "mobile",
    "sites",
    "roles",
]


def import_users(
    path: str,
    resource_name: Optional[str] = None,
    send_email_to_user: Optional[Any] = None,
    resend_as_newly_created: Optional[Any] = None,
    verbose: Optional[Any] = None,
    export_to_file: Optional[Any] = None,
    limit_to_username: Optional[str] = None,
    **kwargs,
):
    """Import users from a CSV file with columns:
    username
    password
    is_staff
    is_active
    first_name
    last_name
    job_title
    email
    mobile
    alternate_email
    site_names: a comma-separated list of sites
    role_names: a comma-separated list of roles
    """
    path = os.path.expanduser(path)
    with open(path) as f:
        reader = csv.DictReader(f, delimiter="|")
        for csv_data in reader:
            opts: dict = {}
            if "username" not in csv_data:
                raise UserImporterError("Invalid file format. Check delimiter")
            if limit_to_username and limit_to_username != csv_data.get("username"):
                continue
            opts.update(username=csv_data.get("username"))
            opts.update(site_names="")
            if csv_data.get("site_names"):
                opts.update(site_names=csv_data.get("site_names").lower().split(","))
            opts.update(role_names="")
            if csv_data.get("role_names"):
                opts.update(role_names=csv_data.get("role_names").lower().split(","))
            opts.update(first_name=csv_data.get("first_name"))
            opts.update(last_name=csv_data.get("last_name"))
            opts.update(mobile=csv_data.get("mobile"))
            opts.update(email=csv_data.get("email"))
            opts.update(alternate_email=csv_data.get("alternate_email"))
            opts.update(job_title=csv_data.get("job_title"))
            opts.update(is_active=True if csv_data.get("is_active") == "True" else False)
            opts.update(is_staff=True if csv_data.get("is_staff") == "True" else False)
            UserImporter(
                resource_name=resource_name,
                send_email_to_user=send_email_to_user,
                verbose=verbose,
                resend_as_newly_created=resend_as_newly_created,
                **opts,
                **kwargs,
            )
    if export_to_file:
        export_users(f"edc_users_imported_{datetime.now().strftime('%Y%m%d%H%M%S')}")


class UserImporter:
    resource_name = "example.com"
    password_nwords = 4
    created_email_template = add_user_template
    updated_email_template = change_user_template

    def __init__(
        self,
        username: str = None,
        first_name: str = None,
        last_name: str = None,
        job_title: str = None,
        email: str = None,
        alternate_email: str = None,
        mobile: str = None,
        site_names: List[str] = None,
        role_names: List[str] = None,
        is_active: bool = None,
        is_staff: bool = None,
        resource_name: str = None,
        created_email_template: Optional[Template] = None,
        updated_email_template: Optional[Template] = None,
        send_email_to_user: Optional[Any] = None,
        test_email_address: Optional[Any] = None,
        resend_as_new: Optional[Any] = None,
        nwords: Optional[int] = None,
        **kwargs,
    ):
        self._messages: List[str] = []
        self._user = None
        self.created = False
        self.password = None
        try:
            self.email, self.alternate_email = email.split(",")
        except (ValueError, AttributeError):
            self.email = email
            self.alternate_email = alternate_email
        self.first_name = first_name
        self.job_title = job_title or "staff member"
        self.last_name = last_name
        self.mobile = mobile
        self.is_active = is_active or False
        self.is_staff = is_staff or False
        self.password_generator = PasswordGenerator(
            nwords=nwords or self.password_nwords, **kwargs
        )
        self.resource_name = resource_name or self.resource_name
        self.resend_as_newly_created = resend_as_new
        self.test_email_address = test_email_address
        self.username = username or self.get_username()
        self.created_email_template = created_email_template or self.created_email_template
        self.updated_email_template = updated_email_template or self.updated_email_template
        self.validate_username()
        if ACCOUNT_MANAGER_ROLE in role_names:
            site_names = [s.name for s in Site.objects.all()]
        self.update_user_sites(site_names or [])
        self.update_user_roles(role_names or [STAFF_ROLE])
        self.user.save()
        self.user.userprofile.job_title = self.job_title
        self.user.userprofile.mobile = self.mobile
        self.user.userprofile.alternate_email = self.alternate_email
        self.user.userprofile.save()

        self.site_names = (
            "\n  - ".join([s.name for s in self.user.userprofile.sites.all()])
            or "(You have not been granted access to any sites)"
        )
        self.role_names = "\n  - ".join(
            [g.display_name for g in self.user.userprofile.roles.all()]
        )
        self.project_name = Protocol().protocol_name
        if send_email_to_user:
            try:
                self.email_message.send(fail_silently=False)
            except ConnectionRefusedError:
                print(self.email_message.body)

    def validate_username(self) -> None:
        if not self.username:
            raise UserImporterError(f"Invalid username. Got username={self.username}")
        if not re.match(r"^\w+$", self.username):
            raise UserImporterError(f"Invalid username. Got {self.username}")

    @property
    def user(self) -> User:
        if not self._user:
            try:
                self._user = User.objects.get(username=self.username)
            except ObjectDoesNotExist:
                self._user = User.objects.create(
                    username=self.username,
                    first_name=self.first_name,
                    last_name=self.last_name,
                    email=self.email,
                    is_staff=self.is_staff,
                    is_active=self.is_active,
                    is_superuser=False,
                )
                self.created = True
            else:
                self._user.first_name = self.first_name
                self._user.last_name = self.last_name
                self._user.email = self.email
                self._user.is_staff = self.is_staff
                self._user.is_active = self.is_active
            finally:
                self.password = self.password_generator.get_password()
                self._user.set_password(self.password)
                self._user.save()
        return self._user

    def update_user_sites(self, site_names: List[str]) -> None:
        self.user.userprofile.sites.clear()
        for site_name in site_names:
            try:
                site = Site.objects.get(name=site_name)
            except ObjectDoesNotExist:
                sites = [s.name for s in Site.objects.all()]
                raise UserImporterError(
                    f"Unknown site for user. Expected one of {sites}. "
                    f"Got {self.user.username}, {site_name}."
                )
            else:
                self.user.userprofile.sites.add(site)

    def update_user_roles(self, role_names: List[str]) -> None:
        self.user.userprofile.roles.clear()
        role_names.extend(required_role_names)
        for role_name in role_names:
            try:
                role = Role.objects.get(name__iexact=role_name)
            except ObjectDoesNotExist:
                raise UserImporterError(
                    f"Unknown role for user. Got role `{role_name}` "
                    f"for user `{self.user.username}`"
                )
            else:
                self.user.userprofile.roles.add(role)

    @property
    def email_message(self) -> EmailMessage:
        if self.created or self.resend_as_newly_created:
            body = self.created_email_template
        else:
            body = self.updated_email_template
        return EmailMessage(
            f"{self.project_name}: Your {self.resource_name} user account is ready.",
            body=body.safe_substitute(self.__dict__),
            from_email="noreply@clinicedc.org",
            to=(self.test_email_address or self.user.email,),
        )

    def get_username(self) -> Optional[str]:
        if self.first_name and self.last_name:
            last_name = "".join(self.last_name.split(" ")).lower()
            return f"{self.first_name.lower()[0]}{last_name}"
        return None



from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.test import override_settings
from django.test.client import RequestFactory

from edc_auth.auth_objects import CLINICIAN_ROLE
from edc_auth.auth_updater import AuthUpdater
from edc_auth.tests.utils import EdcAuthTestCase

from ...backends import ModelBackendWithSite
from ...models.role import Role


@override_settings(
    EDC_AUTH_SKIP_SITE_AUTHS=False,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
)
class TestUserProfile(EdcAuthTestCase):
    def test_default_user_profile_created_by_signal(self):
        """Assert creates userprofile instance."""
        self.user = User.objects.create(username="noam")

    def test_backend_superuser(self):
        """Superuser can always log in."""
        user = User.objects.create(
            username="erik", is_superuser=True, is_active=True, is_staff=True
        )
        user.set_password("password")
        user.save()
        request = RequestFactory()
        backend = ModelBackendWithSite()
        self.assertIsNotNone(
            backend.authenticate(request, username="erik", password="password")  # nosec B106
        )

    @override_settings(SITE_ID=10)
    def test_backend_one_site(self):
        """User of site 10 can log in to site 10."""
        # Site.objects.all().delete()
        ten = Site.objects.get(id=10)
        user = User.objects.create(
            username="erik", is_superuser=False, is_active=True, is_staff=True
        )
        user.set_password("password")
        user.save()
        user.userprofile.sites.add(ten)
        request = RequestFactory()
        backend = ModelBackendWithSite()
        self.assertIsNotNone(
            backend.authenticate(request, username="erik", password="password")  # nosec B106
        )

    @override_settings(SITE_ID=20)
    def test_backend_one_site2(self):
        """User of site 10 cannot log in to site 20."""
        ten = Site.objects.get(id=10)
        user = User.objects.create(
            username="erik", is_superuser=False, is_active=True, is_staff=True
        )
        user.set_password("password")
        user.save()
        user.userprofile.sites.add(ten)
        request = RequestFactory()
        backend = ModelBackendWithSite()
        self.assertIsNone(
            backend.authenticate(  # nosec B106
                request,
                username="erik",
                password="password",
            )
        )

    def test_add_groups_for_role(self):
        AuthUpdater(verbose=False)
        user = User.objects.create(
            username="erik", is_superuser=False, is_active=True, is_staff=True
        )
        self.assertEqual(user.groups.all().count(), 0)
        role = Role.objects.get(name=CLINICIAN_ROLE)
        user.userprofile.roles.add(role)
        user.refresh_from_db()
        self.assertGreater(user.groups.all().count(), 0)



from typing import List, Optional

from django.apps import apps as django_apps


def get_clinic_codenames(
    *crud_apps: str,
    list_app: Optional[str] = None,
    autocomplete_models: Optional[List[str]] = None,
) -> List[str]:
    """Prepares and returns an ordered list of codenames for the
    common edc project apps to be used in `auth_objects`.

    For example, in `auths.py`:

        clinic_codenames = get_clinic_codenames(
            "meta_prn", "meta_subject", "meta_consent", list_app="meta_lists"
        )
    """

    clinic_codenames: List[str] = []
    autocomplete_models = autocomplete_models or []
    try:
        app_config = django_apps.get_app_config(list_app)
    except LookupError:
        pass
    else:
        for model_cls in app_config.get_models():
            model_name: str = model_cls._meta.model_name
            for prefix in ["view"]:
                clinic_codenames.append(f"{app_config.name}.{prefix}_{model_name}")
    for name in crud_apps:
        try:
            app_config = django_apps.get_app_config(name)
        except LookupError:
            pass
        else:
            for model_cls in app_config.get_models():
                label_lower: str = model_cls._meta.label_lower
                model_name: str = model_cls._meta.model_name
                if "historical" in label_lower:
                    clinic_codenames.append(f"{app_config.name}.view_{model_name}")
                elif label_lower in autocomplete_models:
                    clinic_codenames.append(f"{app_config.name}.view_{model_name}")
                else:
                    clinic_codenames.append(f"{app_config.name}.view_{model_name}")
                    for prefix in ["add_", "change_", "delete_"]:
                        clinic_codenames.append(f"{app_config.name}.{prefix}{model_name}")
    clinic_codenames.sort()
    return clinic_codenames



from string import Template

from django.contrib.auth.models import User
from django.core import mail
from django.test import override_settings
from edc_protocol import Protocol
from faker import Faker

from edc_auth.auth_objects import CLINIC, CLINICIAN_ROLE
from edc_auth.auth_updater import AuthUpdater
from edc_auth.import_users import UserImporter, UserImporterError, import_users
from edc_auth.password_setter import PasswordSetter

from ..utils import EdcAuthTestCase, create_user_csv_file, create_users

fake = Faker()

site_names = ["harare", "gaborone", "kampala"]


@override_settings(
    EDC_AUTH_SKIP_SITE_AUTHS=False,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
)
class TestUser(EdcAuthTestCase):
    def setUp(self):
        self.filename = create_user_csv_file(user_count=2, include_passwords=True)

    def test_import_users(self):
        AuthUpdater(verbose=False)
        # import new users
        import_users(self.filename, resource_name=None, send_email_to_user=True)
        self.assertEqual(len(mail.outbox), User.objects.all().count())  # noqa
        self.assertEqual(
            mail.outbox[0].subject,
            f"{Protocol().project_name}: Your example.com user account is ready.",
        )

        # update existing users
        import_users(self.filename, resource_name=None, send_email_to_user=True)
        user_count = User.objects.all().count()
        self.assertEqual(len(mail.outbox), user_count * 2)  # noqa
        self.assertEqual(
            mail.outbox[0].subject,
            f"{Protocol().project_name}: Your example.com user account is ready.",
        )

    def test_bad_username(self):
        AuthUpdater(verbose=False)
        self.assertRaises(
            UserImporterError,
            UserImporter,
            username=None,
            first_name=fake.first_name(),
            last_name=None,
            email=fake.email(),
            site_names=[],
            role_names=[],
            send_email_to_user=True,
        )

        self.assertRaises(
            UserImporterError,
            UserImporter,
            username="erik@",
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            email=fake.email(),
            site_names=[],
            role_names=[],
            send_email_to_user=True,
        )

    def test_unknown_site(self):
        AuthUpdater(verbose=False)
        self.assertRaises(
            UserImporterError,
            UserImporter,
            username="erik",
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            email=fake.email(),
            site_names=["blah"],
            role_names=["CLINICIAN_ROLE"],
            send_email_to_user=True,
        )

    def test_unknown_role(self):
        AuthUpdater(verbose=False)
        self.assertRaises(
            UserImporterError,
            UserImporter,
            username="erik",
            first_name=fake.first_name(),
            last_name=fake.last_name(),
            email=fake.email(),
            site_names=["harare"],
            role_names=["blah"],
            send_email_to_user=True,
        )

    def test_with_custom_templates(self):
        AuthUpdater(verbose=False)
        created_email_template = Template("Hi $first_name, \n\nStay Classy")
        updated_email_template = Template("Hi $first_name, \n\nYou stay classy San Diego")
        first_name = fake.first_name()
        UserImporter(
            username="erik",
            first_name=first_name,
            last_name=fake.last_name(),
            email=fake.email(),
            site_names=["harare"],
            role_names=[CLINICIAN_ROLE],
            send_email_to_user=True,
            created_email_template=created_email_template,
            updated_email_template=updated_email_template,
        )
        self.assertEqual(
            mail.outbox[0].body,
            created_email_template.safe_substitute(first_name=first_name),
        )
        UserImporter(
            username="erik",
            first_name=first_name,
            last_name=fake.last_name(),
            email=fake.email(),
            site_names=["harare"],
            role_names=[CLINICIAN_ROLE],
            send_email_to_user=True,
            created_email_template=created_email_template,
            updated_email_template=updated_email_template,
        )
        self.assertEqual(
            mail.outbox[1].body,
            updated_email_template.safe_substitute(first_name=first_name),
        )

    def test_password_setter_all(self):
        AuthUpdater(verbose=False)
        create_users(5)
        user = User.objects.all()[0]
        pwsetter = PasswordSetter(super_username=user.username)
        pwsetter.reset_all()
        self.assertEqual(len(mail.outbox), User.objects.all().count())  # noqa

    def test_password_setter_groups(self):
        AuthUpdater(verbose=False)
        count = User.objects.filter(groups__name=CLINIC).count()
        create_users(5, group_name=CLINIC)
        user = User.objects.all()[0]
        pwsetter = PasswordSetter(super_username=user.username)
        pwsetter.reset_by_groups([CLINIC])
        self.assertEqual(len(mail.outbox), User.objects.all().count() + count)  # noqa

    def test_password_setter_sites(self):
        AuthUpdater(verbose=False)
        count = User.objects.filter(userprofile__sites__name="harare").count()
        create_users(5, site_name="harare")
        user = User.objects.all()[0]
        pwsetter = PasswordSetter(super_username=user.username)
        pwsetter.reset_by_sites(["harare"])
        self.assertEqual(len(mail.outbox), User.objects.all().count() + count)  # noqa

    def test_password_setter_user(self):
        AuthUpdater(verbose=False)
        usernames = create_users(5)
        user = User.objects.all()[0]
        pwsetter = PasswordSetter(super_username=user.username)
        pwsetter.reset_users(usernames)
        self.assertEqual(len(mail.outbox), 5)  # noqa



from django.core.management.base import BaseCommand

from edc_auth.import_users import import_users


class Command(BaseCommand):

    help = "Import users from a CSV file"

    def add_arguments(self, parser):
        parser.add_argument(
            "--csvfile", default="users.csv", dest="csvfile", help="CSV filename"
        )
        parser.add_argument(
            "--notify-to-test-email",
            default="",
            dest="notify_to_test_email",
            help="Email address to use when notifying users (for testing)",
        )

        parser.add_argument(
            "--resend-as-new",
            default=False,
            action="store_true",
            dest="resend_as_new",
            help="Send email to user as if newly created",
        )

        parser.add_argument(
            "--resource-name",
            type=str,
            default="clinicedc.org",
            dest="resource_name",
            help="Mail server domain",
        )

        parser.add_argument(
            "--verbose",
            default=False,
            action="store_true",
            dest="verbose",
            help="Verbose mode",
        )

        parser.add_argument(
            "--export",
            default=False,
            action="store_true",
            dest="export_to_file",
            help="Export new users to file",
        )

        parser.add_argument(
            "--notify-users",
            default=False,
            action="store_true",
            dest="notify_users",
            help="Send email to users",
        )

        parser.add_argument(
            "--limit_to_user",
            default="",
            type=str,
            dest="limit_to_username",
            help="Limit import to a single username",
        )

    def handle(self, *args, **options):

        import_users(
            options["csvfile"],
            resource_name=options["resource_name"],
            send_email_to_user=options["notify_users"],
            verbose=options["verbose"],
            export_to_file=options["export_to_file"],
            test_email_address=options["notify_to_test_email"],
            resend_as_new=options["resend_as_new"],
            limit_to_username=options["limit_to_username"],
        )



from django.contrib import admin
from django.contrib.auth.models import Group
from django.utils.html import format_html
from edc_model_admin.mixins import TemplatesModelAdminMixin

from edc_auth.admin_site import edc_auth_admin

admin.site.unregister(Group)


@admin.register(Group, site=edc_auth_admin)
class GroupAdmin(TemplatesModelAdminMixin, admin.ModelAdmin):

    ordering = ("name",)

    list_display_links = ("name", "codenames")

    list_display = ("name", "codenames")

    list_filter = (
        "name",
        "permissions__codename",
    )

    search_fields = (
        "name",
        "permissions__codename",
    )

    @staticmethod
    def codenames(obj=None) -> str:
        codenames = []
        for permission in obj.permissions.all():
            codenames.append(permission.codename)
        codenames.sort()
        return format_html("<BR>".join(codenames))



# Generated by Django 2.2.6 on 2019-10-26 15:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0013_auto_20191026_0055")]

    operations = [
        migrations.RenameField(
            model_name="userprofile", old_name="user_roles", new_name="roles"
        )
    ]



import sys

from django.apps import AppConfig as DjangoAppConfig
from django.core.checks.registry import register
from django.core.management.color import color_style
from django.db.models.signals import post_migrate

from .auth_updater import AuthUpdater
from .auth_updater.group_updater import CodenameDoesNotExist
from .system_checks import edc_check

style = color_style()


def post_migrate_user_groups_and_roles(sender=None, **kwargs):  # noqa
    """Update Groups, Role model with EDC defaults."""
    from django.apps import apps as django_apps

    try:
        AuthUpdater(apps=django_apps, verbose=True)
    except CodenameDoesNotExist as e:
        sys.stdout.write(style.ERROR(f"{e}. "))
        sys.stdout.write(
            style.ERROR(
                "\n\nIf the codename is mispelled, correct the error and try again.\n\n"
            )
        )
        sys.stdout.write(
            style.ERROR(
                "\nIf the codename is a custom codename (not Django's), "
                "make sure you created it.\n\n"
            )
        )
        sys.stdout.write(
            style.ERROR(
                "\nIf this is happening in a migration that is creating a new model,\n"
                "the post_migrate signal that creates the new model's permissions\n"
                "might be queued to run AFTER edc_auth's post_migrate signal. Let this\n"
                "migration complete and then run `migrate` again.\n\n"
            )
        )


class AppConfig(DjangoAppConfig):
    name = "edc_auth"
    verbose_name = "Edc Authentication"

    def ready(self):
        from .site_auths import site_auths

        sys.stdout.write(f"Loading {self.verbose_name} ...\n")
        site_auths.autodiscover()
        post_migrate.connect(post_migrate_user_groups_and_roles, sender=self)
        register(edc_check)
        sys.stdout.write(f" Done loading {self.verbose_name}.\n")



from django.conf import settings
from django.contrib.auth.views import LogoutView as BaseLogoutView
from django_revision.revision import site_revision
from edc_protocol import Protocol


class LogoutView(BaseLogoutView):
    next_page = "edc_auth/login"

    @property
    def extra_context(self):
        protocol = Protocol()
        try:
            live_system = settings.LIVE_SYSTEM
        except AttributeError:
            live_system = None
        return {
            "DEBUG": settings.DEBUG,
            "copyright": protocol.copyright,
            "disclaimer": protocol.disclaimer,
            "institution": protocol.institution,
            "license": protocol.license,
            "revision": site_revision.tag,
            "project_name": protocol.project_name,
            "live_system": live_system,
            "INDEX_PAGE": getattr(settings, "INDEX_PAGE", None),
            "INDEX_PAGE_LABEL": getattr(settings, "INDEX_PAGE_LABEL", None),
        }



from django.conf import settings
from django.contrib.auth import views
from django.urls.conf import path

from .views import LoginView, LogoutView

app_name = "edc_auth"

allow_password_reset = getattr(settings, "ALLOW_PASSWORD_RESET", None)

urlpatterns = [
    path("login/", LoginView.as_view(), name="login"),
    path(
        "logout/",
        LogoutView.as_view(template_name="edc_dashboard/login.html"),
        name="logout",
    ),
]

if allow_password_reset:
    urlpatterns += [
        path(
            "password_change/",
            views.PasswordChangeView.as_view(),
            name="password_change",
        ),
        path(
            "password_change/done/",
            views.PasswordChangeDoneView.as_view(),
            name="password_change_done",
        ),
        path("password_reset/", views.PasswordResetView.as_view(), name="password_reset"),
        path(
            "password_reset/done/",
            views.PasswordResetDoneView.as_view(),
            name="password_reset_done",
        ),
        path(
            "reset/<uidb64>/<token>/",
            views.PasswordResetConfirmView.as_view(),
            name="password_reset_confirm",
        ),
        path(
            "reset/done/",
            views.PasswordResetCompleteView.as_view(),
            name="password_reset_complete",
        ),
    ]
else:
    urlpatterns += [
        path(
            "password_reset/",
            LogoutView.as_view(template_name="edc_dashboard/login.html"),
            name="password_reset",
        ),
        path(
            "password_reset/done/",
            LogoutView.as_view(template_name="edc_dashboard/login.html"),
            name="password_reset_done",
        ),
        path(
            "reset/<uidb64>/<token>/",
            LogoutView.as_view(template_name="edc_dashboard/login.html"),
            name="password_reset_confirm",
        ),
        path(
            "reset/done/",
            LogoutView.as_view(template_name="edc_dashboard/login.html"),
            name="password_reset_complete",
        ),
    ]



from copy import copy, deepcopy
from importlib import import_module
from typing import List

from django.contrib.auth.models import Group, Permission
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, override_settings
from edc_randomization.auth_objects import (
    RANDO_BLINDED,
    RANDO_UNBLINDED,
    get_rando_permissions_tuples,
)
from edc_randomization.randomizer import Randomizer
from edc_randomization.site_randomizers import site_randomizers

from edc_auth.auth_updater import AuthUpdater
from edc_auth.site_auths import site_auths

from ...auth_objects import default_groups
from ..randomizers import CustomRandomizer


@override_settings(
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
)
class TestAuthUpdater2(TestCase):
    def test_add_group(self):
        codenames = [
            "edc_auth.add_testmodel",
            "edc_auth.change_testmodel",
            "edc_auth.delete_testmodel",
            "edc_auth.view_testmodel",
        ]
        site_auths.clear()
        site_auths.add_group(*codenames, name="GROUP")
        AuthUpdater()
        group = Group.objects.get(name="GROUP")
        self.assertEqual(
            [p.codename for p in group.permissions.filter(content_type__app_label="edc_auth")],
            [c.split(".")[1] for c in codenames],
        )

    def test_add_group_with_callable(self):
        def codenames_callable() -> List[str]:
            return [
                "edc_auth.add_subjectrequisition",
                "edc_auth.change_subjectrequisition",
                "edc_auth.delete_subjectrequisition",
                "edc_auth.view_subjectrequisition",
            ]

        codenames = [
            "edc_auth.add_testmodel",
            "edc_auth.change_testmodel",
            "edc_auth.delete_testmodel",
            "edc_auth.view_testmodel",
        ]
        codenames_with_callable: list = copy(codenames)
        codenames_with_callable.append(codenames_callable)
        site_auths.clear()
        site_auths.add_group(*codenames_with_callable, name="GROUP")
        AuthUpdater()
        group = Group.objects.get(name="GROUP")

        codenames.extend(codenames_callable())
        codenames.sort()
        self.assertEqual(
            [
                p.codename
                for p in group.permissions.filter(content_type__app_label="edc_auth").order_by(
                    "codename"
                )
            ],
            [c.split(".")[1] for c in codenames],
        )

    def test_add_group_view_only(self):
        codenames = [
            "edc_auth.add_testmodel",
            "edc_auth.change_testmodel",
            "edc_auth.delete_testmodel",
            "edc_auth.view_testmodel",
        ]
        site_auths.clear()
        site_auths.add_group(*codenames, name="GROUP_VIEW_ONLY", view_only=True)
        AuthUpdater()
        group = Group.objects.get(name="GROUP_VIEW_ONLY")
        self.assertEqual(
            [p.codename for p in group.permissions.filter(content_type__app_label="edc_auth")],
            ["view_testmodel"],
        )

    def test_add_group_view_only_with_callable(self):
        def more_codenames():
            return [
                "edc_auth.add_subjectrequisition",
                "edc_auth.change_subjectrequisition",
                "edc_auth.delete_subjectrequisition",
                "edc_auth.view_subjectrequisition",
            ]

        codenames = [
            "edc_auth.add_testmodel",
            "edc_auth.change_testmodel",
            "edc_auth.delete_testmodel",
            "edc_auth.view_testmodel",
            more_codenames,
        ]
        site_auths.clear()
        site_auths.add_group(*codenames, name="GROUP_VIEW_ONLY", view_only=True)
        AuthUpdater()
        group = Group.objects.get(name="GROUP_VIEW_ONLY")
        self.assertEqual(
            [p.codename for p in group.permissions.filter(content_type__app_label="edc_auth")],
            ["view_subjectrequisition", "view_testmodel"],
        )

    def test_add_group_convert_to_export_with_callable(self):
        def more_codenames():
            return [
                "edc_auth.add_subjectrequisition",
                "edc_auth.change_subjectrequisition",
                "edc_auth.delete_subjectrequisition",
                "edc_auth.view_subjectrequisition",
            ]

        codenames = [
            "edc_auth.add_testmodel",
            "edc_auth.change_testmodel",
            "edc_auth.delete_testmodel",
            "edc_auth.view_testmodel",
            more_codenames,
        ]
        site_auths.clear()
        site_auths.add_group(*codenames, name="GROUP_EXPORT", convert_to_export=True)
        AuthUpdater()
        group = Group.objects.get(name="GROUP_EXPORT")
        self.assertEqual(
            [p.codename for p in group.permissions.filter(content_type__app_label="edc_auth")],
            ["export_subjectrequisition", "export_testmodel"],
        )

    def test_add_group_remove_delete_with_callable(self):
        def more_codenames():
            return [
                "edc_auth.add_subjectrequisition",
                "edc_auth.change_subjectrequisition",
                "edc_auth.delete_subjectrequisition",
                "edc_auth.view_subjectrequisition",
            ]

        codenames = [
            "edc_auth.add_testmodel",
            "edc_auth.change_testmodel",
            "edc_auth.delete_testmodel",
            "edc_auth.view_testmodel",
            more_codenames,
        ]
        site_auths.clear()
        site_auths.add_group(*codenames, name="GROUP_NO_DELETE", no_delete=True)
        AuthUpdater()
        group = Group.objects.get(name="GROUP_NO_DELETE")

        codenames = [
            "add_subjectrequisition",
            "add_testmodel",
            "change_subjectrequisition",
            "change_testmodel",
            "view_subjectrequisition",
            "view_testmodel",
        ]
        self.assertEqual(
            [
                p.codename
                for p in group.permissions.filter(content_type__app_label="edc_auth").order_by(
                    "codename"
                )
            ],
            codenames,
        )


@override_settings(
    EDC_AUTH_SKIP_SITE_AUTHS=False,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
)
class TestAuthUpdater(TestCase):
    @classmethod
    def setUpTestData(cls):
        site_randomizers._registry = {}
        site_randomizers.register(Randomizer)
        site_randomizers.register(CustomRandomizer)
        site_auths.initialize()
        import_module("edc_navbar.auths")
        import_module("edc_dashboard.auths")
        import_module("edc_review_dashboard.auths")
        import_module("edc_randomization.auths")

    def test_rando_tuples(self):
        """Given the two registered randomizers, assert view codenames are returned"""
        AuthUpdater(verbose=False)
        self.assertIn(
            ("edc_randomization.view_randomizationlist", "Can view randomization list"),
            get_rando_permissions_tuples(),
        )

        self.assertIn(
            (
                "edc_auth.view_customrandomizationlist",
                "Can view custom randomization list",
            ),
            get_rando_permissions_tuples(),
        )

    def test_removes_for_apps_not_installed_by_exact_match(self):
        """The app edc_action_blah is not installed, and will
        be removed."""
        groups = deepcopy(default_groups)
        groups.update(
            {
                "ACTION_GROUP": [
                    "edc_action_blah.view_actionitem",
                    "edc_action_item.view_actionitem",
                ]
            }
        )
        AuthUpdater(groups=groups)
        groups = Group.objects.get(name="ACTION_GROUP")
        try:
            groups.permissions.get(
                content_type__app_label="edc_action_item", codename="view_actionitem"
            )
        except ObjectDoesNotExist:
            self.fail("Permission unexpectedly does not exist")

        self.assertRaises(
            ObjectDoesNotExist,
            groups.permissions.get,
            content_type__app_label="edc_action_blah",
            codename="view_actionitem",
        )

    def test_removes_edc_permissions_model_perms(self):
        qs = Permission.objects.filter(
            content_type__app_label="edc_auth",
            codename__in=[
                "add_edcpermissions",
                "change_edcpermissions",
                "view_edcpermissions",
                "delete_edcpermissions",
            ],
        )
        self.assertEqual(qs.count(), 4)
        AuthUpdater()
        for group in Group.objects.all():
            qs = group.permissions.all()
            self.assertNotIn("add_dashboard", "|".join([o.codename for o in qs]))
            self.assertNotIn("change_dashboard", "|".join([o.codename for o in qs]))
            self.assertNotIn("view_dashboard", "|".join([o.codename for o in qs]))
            self.assertNotIn("delete_dashboard", "|".join([o.codename for o in qs]))

    def test_group_has_randomization_list_model_view_perms(self):
        """Assert group has view perms for each randomizer,
        others perms are removed.
        """
        AuthUpdater(verbose=False)
        group = Group.objects.get(name=RANDO_BLINDED)
        qs = group.permissions.all()
        self.assertGreater(qs.count(), 0)
        self.assertIn("view_randomizationlist", "|".join([o.codename for o in qs]))
        group = Group.objects.get(name=RANDO_UNBLINDED)
        qs = group.permissions.all()
        self.assertGreater(qs.count(), 0)
        self.assertIn("view_randomizationlist", "|".join([o.codename for o in qs]))

    def test_randomization_list_model_add_change_delete_perms_removed_everywhere(self):
        AuthUpdater(verbose=False)
        for group in Group.objects.all():
            qs = group.permissions.all()
            self.assertNotIn("add_randomizationlist", "|".join([o.codename for o in qs]))
            self.assertNotIn("change_randomizationlist", "|".join([o.codename for o in qs]))
            self.assertNotIn("delete_randomizationlist", "|".join([o.codename for o in qs]))

    def test_removes_randomization_list_model_perms2(self):
        self.assertIn(
            "view_customrandomizationlist",
            "|".join([o.codename for o in Permission.objects.all()]),
        )
        AuthUpdater(verbose=True)
        Permission.objects.filter(
            content_type__app_label__in=["edc_randomization", "edc_auth"]
        )
        # confirm add_, change_, delete_ codenames for rando
        # does not exist in any groups.
        for group in Group.objects.all():
            qs = group.permissions.all()
            for model_name in ["customrandomizationlist", "randomizationlist"]:
                self.assertNotIn(f"add_{model_name}", "|".join([o.codename for o in qs]))
                self.assertNotIn(f"change_{model_name}", "|".join([o.codename for o in qs]))
                self.assertNotIn(f"delete_{model_name}", "|".join([o.codename for o in qs]))
                if group.name in [RANDO_UNBLINDED, RANDO_BLINDED]:
                    self.assertIn(f"view_{model_name}", "|".join([o.codename for o in qs]))



from pprint import pprint
from typing import Any

from django.contrib.auth.models import Group


def compare_codenames_for_group(group_name=None, expected=None):
    group = Group.objects.get(name=group_name)
    codenames = [p.codename for p in group.permissions.all()]
    new_expected = []
    for c in expected:
        try:
            c = c.split(".")[1]
        except IndexError:
            pass
        new_expected.append(c)

    compared = [c for c in new_expected if c not in codenames]
    if compared:
        print(group.name, "missing from codenames")
        pprint(compared)
    compared = [c for c in codenames if c not in new_expected]
    if compared:
        print(group.name, "extra codenames")
        pprint(compared)


def remove_default_model_permissions_from_edc_permissions(auth_updater: Any, app_label: str):
    for group in auth_updater.group_model_cls.objects.all():
        auth_updater.remove_permissions_by_codenames(
            group=group,
            codenames=[
                f"{app_label}.add_edcpermissions",
                f"{app_label}.change_edcpermissions",
                f"{app_label}.delete_edcpermissions",
                f"{app_label}.view_edcpermissions",
            ],
        )



from django.urls.conf import include, path
from django.views.generic.base import RedirectView
from edc_dashboard.views import AdministrationView

urlpatterns = [
    path("accounts/", include("edc_auth.urls")),
    path("edc_adverse_event/", include("edc_adverse_event.urls")),
    path("edc_dashboard/", include("edc_dashboard.urls")),
    path("edc_export/", include("edc_export.urls")),
    path("edc_lab/", include("edc_lab.urls")),
    path("edc_device/", include("edc_device.urls")),
    path("edc_lab_dashboard/", include("edc_lab_dashboard.urls")),
    path("edc_pharmacy/", include("edc_pharmacy.urls")),
    path("edc_reference/", include("edc_reference.urls")),
    path("edc_auth/", include("edc_auth.urls")),
    path("administration/", AdministrationView.as_view(), name="administration_url"),
    path("edc_visit_schedule/", include("edc_visit_schedule.urls")),
    path("", RedirectView.as_view(), name="home_url"),
]



from edc_model.models import BaseUuidModel

from ..model_mixins import EdcPermissionsModelMixin


class EdcPermissions(EdcPermissionsModelMixin, BaseUuidModel):

    # see edc_auth for permissions attached to this model

    class Meta(EdcPermissionsModelMixin.Meta):
        pass



# Generated by Django 2.1 on 2018-08-04 20:41

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0003_auto_20180803_0532")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="user",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
            ),
        )
    ]



from django.contrib.admin import SimpleListFilter
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site


class SitesListFilter(SimpleListFilter):

    title = "Site"
    parameter_name = "site_name"

    def lookups(self, request, model_admin):
        sites = []
        for site in Site.objects.all().order_by("name"):
            sites.append((site.name, site.name.replace("_", " ").title()))
        return tuple(sites)

    def queryset(self, request, queryset):
        """Returns a queryset if the site name is in the list of sites"""
        qs = None
        if self.value():
            qs = get_user_model().objects.filter(userprofile__sites__name__in=[self.value()])
        return qs



import os
import sys

from django.conf import settings
from django.core.checks import Warning
from django.core.management import color_style

style = color_style()


def edc_check(app_configs, **kwargs):  # noqa
    errors = []
    errors = check_etc_dir(errors)
    errors = check_key_path(errors)
    errors = check_static_root(errors)
    errors = check_site_auths(errors)
    errors = check_auth_updater(errors)
    return errors


def check_etc_dir(errors):
    sys.stdout.write(style.SQL_KEYWORD("check_etc_dir ... \r"))
    try:
        settings.ETC_DIR
    except AttributeError:
        pass
    else:
        if not settings.DEBUG and settings.ETC_DIR and not settings.ETC_DIR.startswith("/etc"):
            errors.append(
                Warning(
                    "Insecure configuration. Use root level etc folder. "
                    f"For example, '/etc/{settings.APP_NAME}/' "
                    f"Got {settings.ETC_DIR}",
                    id="settings.ETC_DIR",
                )
            )
        if settings.ETC_DIR and os.access(settings.ETC_DIR, os.W_OK):
            errors.append(
                Warning(
                    "Insecure configuration. Folder is writeable by this user. "
                    f"Got {settings.ETC_DIR}",
                    id="settings.ETC_DIR",
                )
            )
    sys.stdout.write(style.SQL_KEYWORD("check_etc_dir ... done.\n"))
    return errors


def check_static_root(errors):
    sys.stdout.write(style.SQL_KEYWORD("check_static_root ... \r"))
    try:
        settings.STATIC_ROOT
    except AttributeError:
        pass
    else:
        if settings.STATIC_ROOT and not os.path.exists(settings.STATIC_ROOT):
            errors.append(
                Warning(
                    f"Folder does not exist. Got {settings.STATIC_ROOT}",
                    id="settings.STATIC_ROOT",
                )
            )
    sys.stdout.write(style.SQL_KEYWORD("check_static_root ... done.\n"))
    return errors


def check_key_path(errors):
    sys.stdout.write(style.SQL_KEYWORD("check_key_path ...\r"))
    try:
        settings.KEY_PATH
    except AttributeError:
        pass
    else:
        if settings.KEY_PATH and os.access(settings.KEY_PATH, os.W_OK):
            errors.append(
                Warning(
                    "Insecure configuration. Folder is writeable by this user. "
                    f"Got {settings.KEY_PATH}",
                    id="settings.KEY_PATH",
                )
            )
    sys.stdout.write(style.SQL_KEYWORD("check_key_path ... done.\n"))
    return errors


def check_auth_updater(errors):
    sys.stdout.write(style.SQL_KEYWORD("check_auth_updater ...\r"))
    try:
        settings.EDC_AUTH_SKIP_AUTH_UPDATER
    except AttributeError:
        pass
    else:
        if settings.EDC_AUTH_SKIP_AUTH_UPDATER:
            errors.append(
                Warning(
                    "AuthUpdater did not load. Groups and permissions have not been updated. "
                    "See settings.EDC_AUTH_SKIP_AUTH_UPDATER.",
                    id="settings.EDC_AUTH_SKIP_AUTH_UPDATER",
                )
            )
    sys.stdout.write(style.SQL_KEYWORD("check_auth_updater ... done.\n"))
    return errors


def check_site_auths(errors):
    sys.stdout.write(style.SQL_KEYWORD("check_site_auths ...\r"))
    try:
        settings.EDC_AUTH_SKIP_AUTH_UPDATER
    except AttributeError:
        pass
    else:
        if settings.EDC_AUTH_SKIP_AUTH_UPDATER:
            errors.append(
                Warning(
                    "SiteAuths did not autodiscover. Groups and permissions data not ready "
                    "for AuthUpdater. See settings.EDC_AUTH_SKIP_SITE_AUTHS.",
                    id="settings.EDC_AUTH_SKIP_SITE_AUTHS",
                )
            )
    sys.stdout.write(style.SQL_KEYWORD("check_site_auths ... done.\n"))
    return errors



from typing import Tuple

from django.contrib import admin
from django.utils.html import format_html
from edc_model_admin.mixins import TemplatesModelAdminMixin

from ..admin_site import edc_auth_admin
from ..models import Role


@admin.register(Role, site=edc_auth_admin)
class RoleAdmin(TemplatesModelAdminMixin, admin.ModelAdmin):

    fieldsets = ((None, ({"fields": ("display_name", "name", "display_index", "groups")})),)

    list_display_links: Tuple[str, ...] = ("display_name", "group_list")

    list_display: Tuple[str, ...] = ("display_name", "name", "group_list")

    filter_horizontal: Tuple[str, ...] = ("groups",)

    search_fields: Tuple[str, ...] = ("display_name", "name", "groups__name")

    ordering: Tuple[str, ...] = ("display_index", "display_name")

    list_filter = ("groups__name",)

    @staticmethod
    def group_list(obj=None) -> str:
        group_names = []
        for group in obj.groups.all():
            group_names.append(group.name)
        group_names.sort()
        return format_html("<BR>".join(group_names))
